from V3SProtoComm.core.comm.receiver import Receiver
from V3SProtoComm.core.comm.protocols import vssref_command_pb2

import json
from google.protobuf.json_format import MessageToJson

class RefereeComm(Receiver):
    def __init__(self, referee_ip='224.5.23.2', referee_port=10003):
        super(RefereeComm, self).__init__(referee_ip, referee_port)
        self.receiver_socket.setblocking(False)
        self.last_rcv_data = {}

    def receive(self):
        """
        Receive packet and decode.
        """

        try:
            data = super().receive()
            decoded_data = vssref_command_pb2.VSSRef_Command().FromString(data)

            referee_data = json.loads(MessageToJson(decoded_data))
            self.last_rcv_data = referee_data
        except BlockingIOError:
            referee_data = self.last_rcv_data

        return referee_data


import logging
import socket
import struct

import json
import numpy as np

from V3SProtoComm.core.comm.receiver import Receiver
from V3SProtoComm.core.comm.protocols import packet_pb2, wrapper_pb2
from V3SProtoComm.core.data import FieldData, EntityData
from V3SProtoComm.core.comm.thread_job import Job
from google.protobuf.json_format import MessageToDict


class ProtoVision(Receiver):
    def __init__(self, team_color_yellow: bool,
                 field_data: FieldData = None,
                 vision_ip='224.5.23.2', vision_port=10015):
        super(ProtoVision, self).__init__(vision_ip, vision_port)

        self.team_color_yellow = team_color_yellow
        self.field_data = field_data

    def receive(self):
        data = super().receive()
        return data

    def receive_dict(self):

        data = self.receive()

        if self.receiver_ip == '224.0.0.1':
            try:
                env_msg = packet_pb2.Environment().FromString(data)
                env_dict = MessageToDict(env_msg.frame)
                logging.debug("Pacote decodificado como SIMULADOR (Environment)")
                return {"type": "simulator", "data": env_dict}
            except Exception as e:
                logging.warning(f"Falha ao decodificar como Environment: {e}")
                return None

        elif self.receiver_ip == '224.5.23.2':
            try:
                ssl_msg = wrapper_pb2.SSL_WrapperPacket().FromString(data)
                ssl_dict = MessageToDict(ssl_msg)
                logging.debug("Pacote decodificado como VISÃO (SSL_WrapperPacket)")
                return {"type": "vision", "data": ssl_dict}
            except Exception as e:
                logging.warning(f"Falha ao decodificar como SSL_WrapperPacket: {e}")
                return None

        else:
            logging.warning(f"IP não reconhecido: {self.receiver_ip}")
            return None

    def receive_field_data(self) -> FieldData:
        """
        Caso queira um FieldData 'novo' a cada chamada, sem sobrescrever o self.field_data.
        """
        data_dict = self.receive_dict()

        rcv_field_data = FieldData()
        if data_dict is None:
            return rcv_field_data  

        if data_dict["type"] == "simulator":
            self._field_data_from_dict(rcv_field_data, data_dict["data"])
        elif data_dict["type"] == "vision":
            self._field_data_from_vision_dict(rcv_field_data, data_dict["data"])

        return rcv_field_data

    def update(self):
        """
        Atualiza self.field_data usando o que chegar (simulador OU visão).
        """
        if self.field_data is None:
            logging.error('FieldData not instantiated', exc_info=True)
            return

        data_dict = self.receive_dict()
        if data_dict is None:
            return

        if data_dict["type"] == "simulator":
            self._field_data_from_dict(self.field_data, data_dict["data"])
        elif data_dict["type"] == "vision":
            self._field_data_from_vision_dict(self.field_data, data_dict["data"])


    def _field_data_from_dict(self, field_data: FieldData, raw_data_dict):

        if self.team_color_yellow:
            team_list_of_dicts = raw_data_dict.get('robotsYellow', [])
            foes_list_of_dicts = raw_data_dict.get('robotsBlue', [])
            rotate_field = True
        else:
            team_list_of_dicts = raw_data_dict.get('robotsBlue', [])
            foes_list_of_dicts = raw_data_dict.get('robotsYellow', [])
            rotate_field = False

        if 'ball' in raw_data_dict:
            self._entity_from_dict(field_data.ball, raw_data_dict['ball'], rotate_field)

        for i in range(len(team_list_of_dicts)):
            self._entity_from_dict(field_data.robots[i], team_list_of_dicts[i], rotate_field)

        for i in range(len(foes_list_of_dicts)):
            self._entity_from_dict(field_data.foes[i], foes_list_of_dicts[i], rotate_field)


    def _field_data_from_vision_dict(self, field_data: FieldData, raw_data_dict):
        """
        Recebe um dicionário que vem do SSL_WrapperPacket (convertido para JSON).
        Ajusta para puxar 'balls', 'robotsYellow', 'robotsBlue', etc.
        """

        if self.team_color_yellow:
            robot_key = "robotsYellow"
            foe_key = "robotsBlue"
            rotate_field = True
        else:
            robot_key = "robotsBlue"
            foe_key = "robotsYellow"
            rotate_field = False

        if "balls" in raw_data_dict and len(raw_data_dict["balls"]) > 0:
            ball_info = raw_data_dict["balls"][0]
            self._entity_from_dict(field_data.ball, ball_info, rotate_field)

        if robot_key in raw_data_dict:
            for i, robot_data in enumerate(raw_data_dict[robot_key]):
                self._entity_from_dict(field_data.robots[i], robot_data, rotate_field)

        if foe_key in raw_data_dict:
            for i, foe_data in enumerate(raw_data_dict[foe_key]):
                self._entity_from_dict(field_data.foes[i], foe_data, rotate_field)


    def _entity_from_dict(self, entity_data: EntityData, data_dict, rotate_field=False):
        multiplier = 1 if not rotate_field else -1
        sum_to_angle = 0 if not rotate_field else np.pi

        x = data_dict.get('x', 0)
        y = data_dict.get('y', 0)
        orientation = data_dict.get('orientation', 0)
        vx = data_dict.get('vx', 0)
        vy = data_dict.get('vy', 0)
        vorientation = data_dict.get('vorientation', 0)

        entity_data.position.x = x * multiplier
        entity_data.position.y = y * multiplier
        entity_data.position.theta = self._assert_angle(orientation + sum_to_angle)
        entity_data.velocity.x = vx * multiplier
        entity_data.velocity.y = vy * multiplier
        entity_data.velocity.theta = vorientation

    def _assert_angle(self, angle):
        angle = angle % (2 * np.pi)
        if angle > np.pi:
            angle -= 2 * np.pi
        return angle


class ProtoVisionThread(Job):
    def __init__(self, team_color_yellow: bool,
                 field_data: FieldData = None,
                 vision_ip='224.0.0.1', vision_port=10015):
        self.vision = ProtoVision(team_color_yellow, field_data, vision_ip, vision_port)
        super(ProtoVisionThread, self).__init__(self.vision.update)





import logging
from typing import List

from V3SProtoComm.core.comm.transmitter import Transmitter
from V3SProtoComm.core.comm.protocols import vssref_placement_pb2
from V3SProtoComm.core.data import EntityData


class ReplacerComm(Transmitter):
    def __init__(self, team_color_yellow: bool, replacer_ip='224.5.23.2', replacer_port=10004):
        super(ReplacerComm, self).__init__(replacer_ip, replacer_port)

        self.team_color_yellow = team_color_yellow


    def transmit(self, packet):
        super().transmit(packet)


    def place_team(self, replacement_list):
        """
        Encode package and transmit.

        Args:
            replacement_list (list): List of tuples with the entity data and it`s id
        """
        packet = self._fill_robot_command_packet(replacement_list)

        self.transmit(packet)


    def _fill_robot_command_packet(self, replacement_list):
        placement_packet = vssref_placement_pb2.VSSRef_Placement()
        placement_packet.world.teamColor = int(self.team_color_yellow)

        for desired_placement in replacement_list:
            entity_data, robot_id = desired_placement

            robot = placement_packet.world.robots.add()
            robot.robot_id = int(robot_id)
            robot.x = entity_data.position.x
            robot.y = entity_data.position.y
            robot.orientation = entity_data.position.theta

        return placement_packet


from V3SProtoComm.core.data import Pose2D
from V3SProtoComm.core.data import EntityData
from V3SProtoComm.core.data import FieldData
from V3SProtoComm.core.command import TeamCommand
from V3SProtoComm.core.command import RobotCommand


# receiver.py
import socket
import struct
from abc import ABC, abstractmethod

class Receiver(ABC):
    def __init__(self, receiver_ip='224.5.23.2', receiver_port=10002):
        """
        Init Client object.
        Extended description of function.
        Parameters
        ----------
        ip : str
            Multicast IP in format '255.255.255.255'.
        port : int
            Port up to 1024.
        """

        self.receiver_ip = receiver_ip
        self.receiver_port = receiver_port
        self.receiver_socket = self._create_socket()

    @abstractmethod
    def receive(self):
        """
        Receive packet.
        """

        data, _ = self.receiver_socket.recvfrom(1024)

        return data


    def _create_socket(self):
        sock = socket.socket(
            socket.AF_INET,
            socket.SOCK_DGRAM,
            socket.IPPROTO_UDP
        )

        sock.setsockopt(
            socket.SOL_SOCKET,
            socket.SO_REUSEADDR, 1
        )

        sock.bind((self.receiver_ip, self.receiver_port))

        mreq = struct.pack(
            "4sl",
            socket.inet_aton(self.receiver_ip),
            socket.INADDR_ANY
        )

        sock.setsockopt(
            socket.IPPROTO_IP,
            socket.IP_ADD_MEMBERSHIP,
            mreq
        )

        return sock


import threading

class Job(threading.Thread):
    def __init__(self, job_function):
        super(Job, self).__init__()
        self.__flag = threading.Event() 
        self.__flag.set() 
        self.__running = threading.Event()
        self.__running.set() 

        self.job_function = job_function

    def run(self):
        while self.__running.isSet():
            self.__flag.wait() 
            self.job_function()

    def pause(self):
        self.__flag.clear() 

    def resume(self):
        self.__flag.set() 

    def stop(self):
        self.__flag.set() 
        self.__running.clear() 


#transmitter.py

import socket
from abc import ABC, abstractmethod

from V3SProtoComm.core.comm.protocols import packet_pb2

class Transmitter(ABC):
    def __init__(self, transmitter_ip='224.0.0.1', transmitter_port=10002):
        """
        Init Client object.
        Extended description of function.
        Parameters
        ----------
        ip : str
            Multicast IP in format '255.255.255.255'.
        port : int
            Port up to 1024.
        """

        self.transmitter_ip = transmitter_ip
        self.transmitter_port = transmitter_port

        # Create socket
        self.transmitter_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    @abstractmethod
    def transmit(self, packet):
        """
        Transmit packet.
        """

        data = packet.SerializeToString()

        self.transmitter_socket.sendto(data, (self.transmitter_ip, self.transmitter_port))


# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vssref_command.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import vssref_common_pb2 as vssref__common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='vssref_command.proto',
  package='VSSRef.ref_to_team',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x14vssref_command.proto\x12\x12VSSRef.ref_to_team\x1a\x13vssref_common.proto\"\xa9\x01\n\x0eVSSRef_Command\x12\x1a\n\x04\x66oul\x18\x01 \x01(\x0e\x32\x0c.VSSRef.Foul\x12 \n\tteamcolor\x18\x02 \x01(\x0e\x32\r.VSSRef.Color\x12&\n\x0c\x66oulQuadrant\x18\x03 \x01(\x0e\x32\x10.VSSRef.Quadrant\x12\x11\n\ttimestamp\x18\x04 \x01(\x01\x12\x1e\n\x08gameHalf\x18\x05 \x01(\x0e\x32\x0c.VSSRef.Halfb\x06proto3')
  ,
  dependencies=[vssref__common__pb2.DESCRIPTOR,])




_VSSREF_COMMAND = _descriptor.Descriptor(
  name='VSSRef_Command',
  full_name='VSSRef.ref_to_team.VSSRef_Command',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='foul', full_name='VSSRef.ref_to_team.VSSRef_Command.foul', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='teamcolor', full_name='VSSRef.ref_to_team.VSSRef_Command.teamcolor', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='foulQuadrant', full_name='VSSRef.ref_to_team.VSSRef_Command.foulQuadrant', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='VSSRef.ref_to_team.VSSRef_Command.timestamp', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gameHalf', full_name='VSSRef.ref_to_team.VSSRef_Command.gameHalf', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=235,
)

_VSSREF_COMMAND.fields_by_name['foul'].enum_type = vssref__common__pb2._FOUL
_VSSREF_COMMAND.fields_by_name['teamcolor'].enum_type = vssref__common__pb2._COLOR
_VSSREF_COMMAND.fields_by_name['foulQuadrant'].enum_type = vssref__common__pb2._QUADRANT
_VSSREF_COMMAND.fields_by_name['gameHalf'].enum_type = vssref__common__pb2._HALF
DESCRIPTOR.message_types_by_name['VSSRef_Command'] = _VSSREF_COMMAND
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VSSRef_Command = _reflection.GeneratedProtocolMessageType('VSSRef_Command', (_message.Message,), dict(
  DESCRIPTOR = _VSSREF_COMMAND,
  __module__ = 'vssref_command_pb2'
  # @@protoc_insertion_point(class_scope:VSSRef.ref_to_team.VSSRef_Command)
  ))
_sym_db.RegisterMessage(VSSRef_Command)


# @@protoc_insertion_point(module_scope)



#control.py
import logging

from V3SProtoComm.core.comm.transmitter import Transmitter
from V3SProtoComm.core.comm.protocols import packet_pb2
from V3SProtoComm.core.comm.protocols import command_pb2
from V3SProtoComm.core.command import TeamCommand
from V3SProtoComm.core.comm.thread_job import Job

class ProtoControl(Transmitter):
    def __init__(self, team_color_yellow: bool, team_command: TeamCommand = None, control_ip='127.0.0.1', control_port=20011):
        super(ProtoControl, self).__init__(control_ip, control_port)

        self.team_color_yellow = team_color_yellow
        self.team_command = team_command


    def transmit(self, packet: packet_pb2.Packet):
        super().transmit(packet)


    def transmit_robot(self, robot_id, left_speed, right_speed):
        """
        Encode package and transmit.
        """

        packet = self._fill_robot_command_packet(robot_id, left_speed, right_speed)

        self.transmit(packet)


    def _fill_robot_command_packet(self, robot_id, left_speed, right_speed):
        cmd_packet = command_pb2.Commands()

        robot = cmd_packet.robot_commands.add() 
        robot.id          = robot_id
        robot.yellowteam  = self.team_color_yellow
        robot.wheel_left  = left_speed
        robot.wheel_right = right_speed

        packet = packet_pb2.Packet()
        packet.cmd.CopyFrom(cmd_packet) 

        return packet

    def transmit_team(self, team_cmd : TeamCommand):
        """
        Encode package and transmit.

        Parameters
        ----------
        team_cmd : core.commands.TeamCommand
            Commands to all robots
        """

        packet = self._fill_team_command_packet(team_cmd)

        self.transmit(packet)


    def update(self):
        """
        Update the transmitted packet with the team_command
        passed in the constructor
        """

        if self.team_command is None:
            logging.error('TeamCommand not instantiated', exc_info=True)
        else:
            self.transmit_team(self.team_command)


    def stop_team(self):
        stop_team_cmd = TeamCommand()
        self.transmit_team(stop_team_cmd)


    def _fill_team_command_packet(self, team_command : TeamCommand):
        cmd_packet = command_pb2.Commands()

        for i in range(len(team_command.commands)):
            cmd = cmd_packet.robot_commands.add()
            cmd.id          = i
            cmd.yellowteam  = self.team_color_yellow
            cmd.wheel_left  = team_command.commands[i].left_speed
            cmd.wheel_right = team_command.commands[i].right_speed

        packet = packet_pb2.Packet()
        packet.cmd.CopyFrom(cmd_packet) 

        return packet


class ProtoControlThread(Job):
    def __init__(self, team_color_yellow: bool, team_command: TeamCommand = None, control_ip='224.5.23.2', control_port=20011):
        self.control = ProtoControl(team_color_yellow, team_command, control_ip, control_port)

        super(ProtoControlThread, self).__init__(self.control.update)


    def pause(self):
        super().pause()
        self.control.stop_team()


    def resume(self):
        super().resume()


    def stop(self):
        super().pause()
        self.control.stop_team()


# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vssref_placement.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import vssref_common_pb2 as vssref__common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='vssref_placement.proto',
  package='VSSRef.team_to_ref',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x16vssref_placement.proto\x12\x12VSSRef.team_to_ref\x1a\x13vssref_common.proto\"0\n\x10VSSRef_Placement\x12\x1c\n\x05world\x18\x01 \x01(\x0b\x32\r.VSSRef.Frameb\x06proto3')
  ,
  dependencies=[vssref__common__pb2.DESCRIPTOR,])




_VSSREF_PLACEMENT = _descriptor.Descriptor(
  name='VSSRef_Placement',
  full_name='VSSRef.team_to_ref.VSSRef_Placement',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='world', full_name='VSSRef.team_to_ref.VSSRef_Placement.world', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=67,
  serialized_end=115,
)

_VSSREF_PLACEMENT.fields_by_name['world'].message_type = vssref__common__pb2._FRAME
DESCRIPTOR.message_types_by_name['VSSRef_Placement'] = _VSSREF_PLACEMENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VSSRef_Placement = _reflection.GeneratedProtocolMessageType('VSSRef_Placement', (_message.Message,), dict(
  DESCRIPTOR = _VSSREF_PLACEMENT,
  __module__ = 'vssref_placement_pb2'
  # @@protoc_insertion_point(class_scope:VSSRef.team_to_ref.VSSRef_Placement)
  ))
_sym_db.RegisterMessage(VSSRef_Placement)


# @@protoc_insertion_point(module_scope)



from V3SProtoComm.core.comm.controls import ProtoControl
from V3SProtoComm.core.comm.controls import ProtoControlThread
from V3SProtoComm.core.comm.vision import ProtoVision
from V3SProtoComm.core.comm.vision import ProtoVisionThread
from V3SProtoComm.core.comm.referee import RefereeComm
from V3SProtoComm.core.comm.thread_job import Job

#ABC
from V3SProtoComm.core.comm.transmitter import Transmitter
from V3SProtoComm.core.comm.receiver import Receiver

from V3SProtoComm.core.comm.protocols import command_pb2
from V3SProtoComm.core.comm.protocols import common_pb2
from V3SProtoComm.core.comm.protocols import packet_pb2
from V3SProtoComm.core.comm.protocols import replacement_pb2
from V3SProtoComm.core.comm.protocols import vssref_command_pb2
from V3SProtoComm.core.comm.protocols import vssref_common_pb2
from V3SProtoComm.core.comm.protocols import vssref_placement_pb2
from V3SProtoComm.core.comm.protocols import wrapper_pb2
from V3SProtoComm.core.comm.protocols import messages_robocup_ssl_detection_pb2
from V3SProtoComm.core.comm.protocols import messages_robocup_ssl_geometry_pb2



# V3SProtoComm/core/data.py

FIELD_WIDTH = 1.3
FIELD_LENGTH = 1.5

GOAL_WIDTH = 0.4
GOAL_DEPTH = 0.1

GOAL_AREA_WIDTH = 0.7
GOAL_AREA_DEPTH = 0.15

ROBOT_SIZE = 0.075
WHEEL_RADIUS = 0.025
BALL_RADIUS = 0.02135


class Pose2D:
    def __init__(self, x: float = 0, y: float = 0, theta: float = 0):
        self.x = x
        self.y = y
        self.theta = theta

    def __str__(self):
        return f'x: {self.x:.02f}  y: {self.y:.02f}  th: {self.theta:.02f}'

    def __repr__(self):
        return f'Pose2D({self})'


class EntityData:
    def __init__(self):
        self.position = Pose2D()
        self.velocity = Pose2D()
        self.confidence = 1.0  
        self.pixel_x = 0.0      
        self.pixel_y = 0.0     

    def __str__(self):
        msg = (
            f'Position: {self.position}\n'
            f'Velocity: {self.velocity}\n'
            f'Confidence: {self.confidence:.02f}\n'
            f'Pixel (x, y): ({self.pixel_x:.02f}, {self.pixel_y:.02f})\n'
        )
        return msg

    def __repr__(self):
        return f'EntityData({self})'


class FieldData:
    def __init__(self, num_robots: int = 3, num_foes: int = 3):
        # Cria listas de EntityData para o time (robots), adversários (foes)
        # e a bola
        self.robots = [EntityData() for _ in range(num_robots)]
        self.foes = [EntityData() for _ in range(num_foes)]
        self.ball = EntityData()

    def __str__(self):
        msg = f'BALL\n{self.ball}'
        for i, robot in enumerate(self.robots):
            msg += f'\nROBOT_{i}\n{robot}'
        for i, foe in enumerate(self.foes):
            msg += f'\nFOE_{i}\n{foe}'
        return msg

    def __repr__(self):
        return f'FieldData({self})'



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vssref_common.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='vssref_common.proto',
  package='VSSRef',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x13vssref_common.proto\x12\x06VSSRef\"D\n\x05Robot\x12\x10\n\x08robot_id\x18\x01 \x01(\r\x12\t\n\x01x\x18\x02 \x01(\x01\x12\t\n\x01y\x18\x03 \x01(\x01\x12\x13\n\x0borientation\x18\x04 \x01(\x01\"H\n\x05\x46rame\x12 \n\tteamColor\x18\x01 \x01(\x0e\x32\r.VSSRef.Color\x12\x1d\n\x06robots\x18\x02 \x03(\x0b\x32\r.VSSRef.Robot*s\n\x04\x46oul\x12\r\n\tFREE_KICK\x10\x00\x12\x10\n\x0cPENALTY_KICK\x10\x01\x12\r\n\tGOAL_KICK\x10\x02\x12\r\n\tFREE_BALL\x10\x03\x12\x0b\n\x07KICKOFF\x10\x04\x12\x08\n\x04STOP\x10\x05\x12\x0b\n\x07GAME_ON\x10\x06\x12\x08\n\x04HALT\x10\x07*\'\n\x05\x43olor\x12\x08\n\x04\x42LUE\x10\x00\x12\n\n\x06YELLOW\x10\x01\x12\x08\n\x04NONE\x10\x02*[\n\x08Quadrant\x12\x0f\n\x0bNO_QUADRANT\x10\x00\x12\x0e\n\nQUADRANT_1\x10\x01\x12\x0e\n\nQUADRANT_2\x10\x02\x12\x0e\n\nQUADRANT_3\x10\x03\x12\x0e\n\nQUADRANT_4\x10\x04*~\n\x04Half\x12\x0b\n\x07NO_HALF\x10\x00\x12\x0e\n\nFIRST_HALF\x10\x01\x12\x0f\n\x0bSECOND_HALF\x10\x02\x12\x17\n\x13OVERTIME_FIRST_HALF\x10\x03\x12\x18\n\x14OVERTIME_SECOND_HALF\x10\x04\x12\x15\n\x11PENALTY_SHOOTOUTS\x10\x05\x62\x06proto3')
)

_FOUL = _descriptor.EnumDescriptor(
  name='Foul',
  full_name='VSSRef.Foul',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FREE_KICK', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PENALTY_KICK', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GOAL_KICK', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FREE_BALL', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KICKOFF', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STOP', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GAME_ON', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HALT', index=7, number=7,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=175,
  serialized_end=290,
)
_sym_db.RegisterEnumDescriptor(_FOUL)

Foul = enum_type_wrapper.EnumTypeWrapper(_FOUL)
_COLOR = _descriptor.EnumDescriptor(
  name='Color',
  full_name='VSSRef.Color',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BLUE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='YELLOW', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NONE', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=292,
  serialized_end=331,
)
_sym_db.RegisterEnumDescriptor(_COLOR)

Color = enum_type_wrapper.EnumTypeWrapper(_COLOR)
_QUADRANT = _descriptor.EnumDescriptor(
  name='Quadrant',
  full_name='VSSRef.Quadrant',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NO_QUADRANT', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QUADRANT_1', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QUADRANT_2', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QUADRANT_3', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QUADRANT_4', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=333,
  serialized_end=424,
)
_sym_db.RegisterEnumDescriptor(_QUADRANT)

Quadrant = enum_type_wrapper.EnumTypeWrapper(_QUADRANT)
_HALF = _descriptor.EnumDescriptor(
  name='Half',
  full_name='VSSRef.Half',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NO_HALF', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FIRST_HALF', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SECOND_HALF', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OVERTIME_FIRST_HALF', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OVERTIME_SECOND_HALF', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PENALTY_SHOOTOUTS', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=426,
  serialized_end=552,
)
_sym_db.RegisterEnumDescriptor(_HALF)

Half = enum_type_wrapper.EnumTypeWrapper(_HALF)
FREE_KICK = 0
PENALTY_KICK = 1
GOAL_KICK = 2
FREE_BALL = 3
KICKOFF = 4
STOP = 5
GAME_ON = 6
HALT = 7
BLUE = 0
YELLOW = 1
NONE = 2
NO_QUADRANT = 0
QUADRANT_1 = 1
QUADRANT_2 = 2
QUADRANT_3 = 3
QUADRANT_4 = 4
NO_HALF = 0
FIRST_HALF = 1
SECOND_HALF = 2
OVERTIME_FIRST_HALF = 3
OVERTIME_SECOND_HALF = 4
PENALTY_SHOOTOUTS = 5



_ROBOT = _descriptor.Descriptor(
  name='Robot',
  full_name='VSSRef.Robot',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='robot_id', full_name='VSSRef.Robot.robot_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='x', full_name='VSSRef.Robot.x', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='VSSRef.Robot.y', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='orientation', full_name='VSSRef.Robot.orientation', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31,
  serialized_end=99,
)


_FRAME = _descriptor.Descriptor(
  name='Frame',
  full_name='VSSRef.Frame',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='teamColor', full_name='VSSRef.Frame.teamColor', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='robots', full_name='VSSRef.Frame.robots', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=101,
  serialized_end=173,
)

_FRAME.fields_by_name['teamColor'].enum_type = _COLOR
_FRAME.fields_by_name['robots'].message_type = _ROBOT
DESCRIPTOR.message_types_by_name['Robot'] = _ROBOT
DESCRIPTOR.message_types_by_name['Frame'] = _FRAME
DESCRIPTOR.enum_types_by_name['Foul'] = _FOUL
DESCRIPTOR.enum_types_by_name['Color'] = _COLOR
DESCRIPTOR.enum_types_by_name['Quadrant'] = _QUADRANT
DESCRIPTOR.enum_types_by_name['Half'] = _HALF
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Robot = _reflection.GeneratedProtocolMessageType('Robot', (_message.Message,), dict(
  DESCRIPTOR = _ROBOT,
  __module__ = 'vssref_common_pb2'
  # @@protoc_insertion_point(class_scope:VSSRef.Robot)
  ))
_sym_db.RegisterMessage(Robot)

Frame = _reflection.GeneratedProtocolMessageType('Frame', (_message.Message,), dict(
  DESCRIPTOR = _FRAME,
  __module__ = 'vssref_common_pb2'
  # @@protoc_insertion_point(class_scope:VSSRef.Frame)
  ))
_sym_db.RegisterMessage(Frame)


# @@protoc_insertion_point(module_scope)



from . import command_pb2
from . import common_pb2
from . import packet_pb2
from . import replacement_pb2
from . import vssref_command_pb2
from . import vssref_common_pb2
from . import vssref_placement_pb2
from . import wrapper_pb2
from . import messages_robocup_ssl_detection_pb2
from . import messages_robocup_ssl_geometry_pb2


from setuptools import setup, find_packages

setup(
    name="V3SProtoComm",
    version="2.1.0",
    description="Pacote para comunicação e controle de robôs via Protobuf",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="Thassio",
    author_email="thxssio@gmail.com",
    url="https://github.com/Taurabots/V3SProtoComm",
    packages=find_packages(),
    include_package_data=True, 
    install_requires=[
        "numpy",
        "protobuf",
        "six",
        "toml",
        "wrapt",
    ],
    entry_points={
        "console_scripts": [
            "gotoball=gotoball:gotoball",
        ],
    },
)



class RobotCommand(object):
    def __init__(self, left_speed=0, right_speed=0):
        self.left_speed = left_speed
        self.right_speed = right_speed

    def __add__(self, other):
        sum_of_left = self.left_speed + other.left_speed
        sum_of_right = self.right_speed + other.right_speed
        return RobotCommand(sum_of_left, sum_of_right)

    def __sub__(self, other):
        sum_of_left = self.left_speed - other.left_speed
        sum_of_right = self.right_speed - other.right_speed
        return RobotCommand(sum_of_left, sum_of_right)

    def __mul__(self, multiplier):
        return RobotCommand(self.left_speed * multiplier, self.right_speed * multiplier)

    def __str__(self):
        msg = (
            f'LEFT SPEED: {self.left_speed}\n'
            f'RIGHT SPEED: {self.right_speed}\n'
        )
        return msg

    def __repr__(self):
        return f'RobotCommand({self})'


class TeamCommand(object):
    def __init__(self):
        self.commands = [RobotCommand() for i in range(3)]

    def reset(self):
        for cmd in self.commands:
            cmd.left_speed = 0
            cmd.right_speed = 0

    def __str__(self):
        msg = f'\nTEAM COMMAND:\n'
        for i in range(3):
            msg += f'ROBOT_{i}\n{self.commands[i]}'
        return msg

    def __repr__(self):
        return f'TeamCommand({self})'